@@ -2,7 +2,8 @@
 #![deny(rust_2018_idioms)]
 #![deny(rust_2021_compatibility)]
 
-use gm_artifacts::PlatformBuilder;
+#[cfg(not(any(target_os = "windows", target_os = "macos")))]
+compile_error!("we only support `windows` and `macos` targets!");
 
 use crate::igor::{OutputKind, TargetFolders};
 
@@ -12,7 +13,6 @@ mod input {
     mod cli;
     mod config_file;
     mod get_input;
-    pub use cli::RunOptions;
     pub use get_input::{parse_inputs, Operation, RunKind};
 }
 mod igor {
@@ -21,50 +21,14 @@ mod igor {
 
     mod build;
     pub use build::*;
-
-    mod user_data;
-    pub use user_data::*;
 }
 
-mod gm_artifacts {
-    mod steam_options;
-    pub use steam_options::*;
-
-    mod macros;
-    pub use macros::*;
-
-    mod preferences;
-    pub use preferences::*;
-
-    mod target_options;
-    pub use target_options::*;
-
-    mod platform;
-    pub use platform::*;
-
-    mod build;
-    pub use build::*;
-}
+mod gm_artifacts;
+pub use gm_artifacts::{DefaultPlatformData, DEFAULT_PLATFORM_DATA, DEFAULT_RUNTIME_NAME};
 mod manifest;
 
-mod runner {
-    mod run;
-    pub use run::{rerun_old, run_command};
-
-    #[cfg(not(target_os = "windows"))]
-    mod invoke_nix;
-    #[cfg(not(target_os = "windows"))]
-    pub(super) use invoke_nix::{invoke_release, invoke_rerun, invoke_run};
-
-    #[cfg(target_os = "windows")]
-    mod invoke_win;
-    #[cfg(target_os = "windows")]
-    pub(super) use invoke_win::{invoke_release, invoke_rerun, invoke_run};
-
-    mod compiler_handler;
-    mod gm_uri_parse;
-    mod printer;
-}
+mod runner;
+pub use runner::{PlatformOptions, RunOptions, TaskOptions};
 
 #[cfg(target_os = "windows")]
 mod trailing_comma_util;
@@ -72,29 +36,31 @@ mod trailing_comma_util;
 fn main() -> AnyResult {
     color_eyre::install()?;
 
-    let (mut options, operation) = input::parse_inputs();
-    options.canonicalize()?;
+    let (mut options, operation) = input::parse_inputs()?;
 
-    // build our platform handle here
-    let platform = {
-        let mut builder = PlatformBuilder::new();
-        if options.beta {
-            builder.set_beta();
-        }
-        if let Some(install) = &options.gms2_install_location {
-            builder.set_app_override(Some(install.to_owned()));
-        }
-        if let Some(runtime) = &options.runtime {
-            builder.set_runtime_name(runtime.to_owned());
-        }
-        if let Some(runtime_full_path) = &options.runtime_location_override {
-            builder.set_runtime_override(Some(runtime_full_path.to_owned()));
-        }
+    if let Err(e) = options.platform.canonicalize() {
+        println!(
+            "{}: {:?}",
+            console::style("adam error").bright().red(),
+            console::style(e).bold()
+        );
 
-        builder.generate()
-    };
+        return Ok(());
+    }
 
-    let application_data = match igor::ApplicationData::new(&options.yyp) {
+    if options.task.yyc {
+        if let Err(e) = options.platform.canonicalize_yyc() {
+            println!(
+                "{}: {}",
+                console::style("adam error").bright().red(),
+                console::style(e).bold()
+            );
+
+            return Ok(());
+        }
+    }
+
+    let application_data = match igor::ApplicationData::new() {
         Ok(v) => v,
         Err(e) => {
             println!(
@@ -107,20 +73,6 @@ fn main() -> AnyResult {
         }
     };
 
-    // check if we can make a user data raw...
-    if let Err(e) = igor::load_user_data(
-        &platform,
-        &mut options.user_license_folder,
-        &mut options.visual_studio_path,
-    ) {
-        println!(
-            "{}: {}\naborting",
-            console::style("adam error").bright().red(),
-            console::style(e).bold()
-        );
-        return Ok(());
-    };
-
     // handle a clean, extract the build_data
     let run_kind = match operation {
         input::Operation::Run(inner) => inner,
@@ -129,7 +81,7 @@ fn main() -> AnyResult {
             if let Err(e) = std::fs::remove_dir_all(
                 application_data
                     .current_directory
-                    .join(options.output_folder()),
+                    .join(&options.task.output_folder),
             ) {
                 println!("{} on clean: {}", console::style("error").bright().red(), e);
             }
@@ -137,8 +89,15 @@ fn main() -> AnyResult {
         }
     };
 
+    // fire any specific behavior to this run kind
+    if run_kind == input::RunKind::Test {
+        for var in options.task.test_env_variables.iter() {
+            std::env::set_var(var, "1");
+        }
+    }
+
     // check if we have a valid yyc bat
-    if options.yyc {
+    if options.task.yyc {
         if cfg!(not(target_os = "windows")) {
             println!(
                 "{}: {}\nPlease log a feature request at https://github.com/NPC-Studio/adam/issues",
@@ -148,7 +107,7 @@ fn main() -> AnyResult {
             return Ok(());
         }
 
-        if let Some(visual_studio_path) = options.visual_studio_path {
+        if options.platform.visual_studio_path.exists() == false {
             println!(
                 "{}: {}.\n\
             Supplied path in preferences was \"{}\" but it did not exist.\n\
@@ -158,14 +117,14 @@ fn main() -> AnyResult {
         https://help.yoyogames.com/hc/en-us/articles/227860547-GMS2-Required-SDKs",
                 console::style("error").bright().red(),
                 console::style("no valid path to visual studio .bat build file").bold(),
-                visual_studio_path.display(),
+                options.platform.visual_studio_path,
             );
 
             return Ok(());
         }
     }
 
-    let output_kind = if options.yyc {
+    let output_kind = if options.task.yyc {
         igor::OutputKind::Yyc
     } else {
         igor::OutputKind::Vm
@@ -174,28 +133,37 @@ fn main() -> AnyResult {
     let build_data = igor::BuildData {
         folders: TargetFolders::new(
             &application_data.current_directory,
-            options.output_folder.as_deref(),
+            options.task.output_folder.as_std_path(),
             output_kind,
             &application_data.project_name,
         )?,
         output_kind,
         project_filename: application_data.project_name,
         project_directory: application_data.current_directory,
-        user_dir: platform.user_data.clone(),
-        license_folder: options.user_license_folder.clone().unwrap(),
-        runtime_location: platform.runtime_location.clone(),
-        target_mask: platform.target_mask,
-        application_path: platform.application_path.clone(),
-        config: options.config.as_deref().unwrap_or("Default").to_owned(),
+        // user_dir: options.platform.user_data.clone(),
+        user_dir: Default::default(),
+        license_folder: options
+            .platform
+            .user_license_folder
+            .as_std_path()
+            .to_owned(),
+        runtime_location: options.platform.runtime_location.as_std_path().to_owned(),
+        // target_mask: options.platform.target_mask,
+        target_mask: 0,
+        application_path: options
+            .platform
+            .gms2_application_location
+            .as_std_path()
+            .to_owned(),
+        config: options.task.config.clone(),
     };
 
     let gm_build = gm_artifacts::GmBuild::new(&build_data);
     let macros = gm_artifacts::GmMacros::new(&build_data);
-    let visual_studio_path = options.visual_studio_path.clone();
+    let visual_studio_path = options.platform.visual_studio_path.clone();
 
     // check if we need to make a new build at all, or can go straight to the runner
-    if options.ignore_cache == 0
-        && cfg!(target_os = "windows")
+    if options.task.ignore_cache == 0
         && manifest::check_manifest(
             build_data.config.clone(),
             &build_data.project_directory,
@@ -221,7 +189,7 @@ fn main() -> AnyResult {
 
     // write in the preferences
     let preferences = if build_data.output_kind == OutputKind::Yyc {
-        gm_artifacts::GmPreferences::new(visual_studio_path.unwrap())
+        gm_artifacts::GmPreferences::new(visual_studio_path.as_std_path().to_owned())
     } else {
         gm_artifacts::GmPreferences::default()
     };
@@ -255,7 +223,27 @@ fn main() -> AnyResult {
     )
     .unwrap();
 
-    runner::run_command(&build_location, macros, options, run_kind);
+    if runner::run_command(&build_location, macros, options, run_kind) {
+        if run_kind.is_test() {
+            println!(
+                "adam test result: {}",
+                console::style("ok").green().bright()
+            );
+        } else {
+            println!("adam {}", console::style("complete").green().bright());
+        }
 
-    Ok(())
+        std::process::exit(0);
+    } else {
+        if run_kind.is_test() {
+            println!(
+                "adam test result: {}",
+                console::style("FAILED").red().bright()
+            );
+        } else {
+            println!("adam {}", console::style("FAILED").red().bright());
+        }
+
+        std::process::exit(1);
+    }
 }
